Package: motifStack
Type: Package
Version: 1.57.0
Title: Plot stacked logos for single or multiple DNA, RNA and amino
        acid sequence
Authors@R: c(person(given="Jianhong", family="Ou", email="jou@morgridge.org", 
        role=c("aut", "cre"), comment=c(ORCID="0000-0002-8652-2488")), 
        person(given="Michael", family="Brodsky", role = "ctb", 
        email = "Michael.Brodsky@umassmed.edu"), 
        person(given="Scot", family="Wolfe", role = "ctb", 
        email = "Scot.Wolfe@umassmed.edu"),
        person("Lihua Julie Zhu", "Developer", role = "aut",
        email = "Julie.Zhu@umassmed.edu"))
Author: Jianhong Ou, Michael Brodsky, Scot Wolfe and Lihua Julie Zhu
Maintainer: Jianhong Ou <jou@morgridge.org>
Depends: R (>= 2.15.1), methods, grid
Imports: ade4, Biostrings, ggplot2, grDevices, graphics, htmlwidgets,
        stats, stats4, utils, XML, TFBSTools
Suggests: Cairo, grImport, grImport2, BiocGenerics, MotifDb,
        RColorBrewer, BiocStyle, knitr, RUnit, rmarkdown, JASPAR2020
biocViews: SequenceMatching, Visualization, Sequencing, Microarray,
        Alignment, ChIPchip, ChIPSeq, MotifAnnotation, DataImport
Description: The motifStack package is designed for graphic 
             representation of multiple motifs with different similarity 
             scores. It works with both DNA/RNA sequence motif and amino 
             acid sequence motif. In addition, it provides the flexibility 
             for users to customize the graphic parameters such as the 
             font type and symbol colors.
License: GPL (>= 2)
Lazyload: yes
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/motifStack
git_branch: devel
git_last_commit: 3f6e3dc
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:55:53 UTC; biocbuild
