# motifStack

[![DOI](https://img.shields.io/badge/DOI-10.1038%2Fnmeth.4555-brightgreen.svg)](https://doi.org/10.1038/nmeth.4555)
[![platforms](http://bioconductor.org/shields/availability/devel/motifStack.svg)](http://bioconductor.org/packages/devel/bioc/html/motifStack.html)
[![build](http://bioconductor.org/shields/build/devel/bioc/motifStack.svg)](http://bioconductor.org/packages/devel/bioc/html/motifStack.html)

Plot stacked logos for single or multiple DNA, RNA and amino acid sequence

The motifStack package is designed for graphic representation of multiple motifs with different similarity scores. It works with both DNA/RNA sequence motif and amino acid sequence motif. In addition, it provides the flexibility for users to customize the graphic parameters such as the font type and symbol colors.

## Installation

To install this package, start R and enter:

```r
library(BiocManager)
BiocManager::install("motifStack")
```

## Documentation

To view documentation of motifStack, start R and enter:
```r
browseVignettes("motifStack")
```

