#' Plot stacked logos for single or multiple DNA, RNA and amino acid sequence
#' 
#' motifStack is a package that is able to draw amino acid sequence as easy as
#' to draw DNA/RNA sequence. motifStack provides the flexibility for users to
#' select the font type and symbol colors. motifStack is designed for graphical
#' representation of multiple motifs.
#' 
#' 
#' @name motifStack-package
#' @aliases motifStack-package
#' @docType package
#' @author Jianhong Ou and Lihua Julie Zhu
#' 
#' Maintainer: Jianhong Ou <jianhong.ou@@umassmed.edu>
#' @keywords package
"_PACKAGE"